import re

import requests

from flask import json

from tek.config import lazy_configurable
from tek.tools import find
from tek import logger


@lazy_configurable(client=['rest_api_port', 'rest_api_url'])
class ApiClient(object):

    def _url(self, path):
        return '{}:{}/{}'.format(self._rest_api_url, self._rest_api_port, path)

    def _request(self, req_type, path, body):
        headers = {'content-type': 'application/json'}
        requester = getattr(requests, req_type)
        return requester(self._url(path), data=json.dumps(body),
                         headers=headers).json().get('response', None)

    def get(self, path, body={}):
        return self._request('get', path, body)

    def post(self, path, body={}):
        return self._request('post', path, body)

    def put(self, path, body={}):
        return self._request('put', path, body)

    def add_link(self, url, series=None, season=None, episode=None):
        if episode is not None:
            data = dict(series=series, season=season, episode=episode)
            _id = self.get('release_id', body=data)
        else:
            dls = self.get('download/pending')
            release = find(lambda r: re.search(r[1], url, re.I), dls)
            _id = release[0] if release else -1
        if _id != -1:
            self.put('release/{}/link'.format(_id), body=dict(url=url))
            logger.info('Added link to release.')
        else:
            logger.error('No matching release found!')
        return _id

    def explain(self):
        output = self.get('release/explain')
        for line in output:
            logger.info(line)
        return output

    def mark_downloaded(self, series, season='', episode=''):
        is_id = series.isdigit()
        if is_id:
            success = self.put('release/{}/downloaded'.format(series))
        else:
            success = self.put('release/{}/{}/{}/downloaded'.format(series,
                                                                    season,
                                                                    episode))
        if success:
            logger.info('Success!')
            return True
        else:
            logger.error('Release not found!')
            return False

    def list(self, regex=''):
        matches = self.get('release', body=dict(regex=regex))
        if matches:
            text = 'id #{}: {}'
            for id, description in matches:
                logger.info(text.format(id, description))
        else:
            logger.info('No matching release found.')
        return matches

    def create_release(self, series, season, episode):
        response = self.post('release/{}/{}/{}'.format(series, season,
                                                       episode))
        logger.info(response)
        return response

__all__ = []
