""" configuration for daemon """


import os

from tek.config import (Configurations, PathConfigOption, ListConfigOption,
                        FileSizeConfigOption)

try:
    from series.config import reset_config as s_reset_config
    __has_parent__ = True
except ImportError:
    __has_parent__ = False


def reset_config(register_files=True, reset_parent=False):
    """ Initialize the config. """
    if __has_parent__ and reset_parent:
        s_reset_config(register_files=register_files,
                       reset_parent=reset_parent)
    path_template = '{name}/s{season}/{name}_{season:0>2}x{episode:0>2}.{ext}'
    Configurations.register_config(
        'series',
        'daemon',
        rss_urls=ListConfigOption(),
        rss_interval=15 * 60,
        download_dir=PathConfigOption(os.getcwd()),
        run_downloader=True,
        run_archiver=True,
        run_subsyncer=True,
        run_feed_poller=True,
        run_rest_api=True,
        use_sharehosters=True,
        use_torrent=False,
        torrent_launcher='transmission-remote -a {}',
        min_size=FileSizeConfigOption(),
        path_template=path_template,
        rest_api_port=8110,
        rest_api_host='0.0.0.0',
        prefer_hosters=ListConfigOption(),
    )
    Configurations.override('store_episode', match_threshold=1.,
                            overwrite=True)
reset_config()
