import time
from threading import Thread
from datetime import datetime, timedelta

import sqlalchemy

from tek import logger
from tek.tools import find

class Handler(Thread):

    def __init__(self, queue, commit, interval, description):
        self._queue = queue
        self._commit = commit
        self._interval = interval
        self._description = description
        super(Handler, self).__init__()
        self._running = True

    def run(self):
        logger.info('Starting {}.'.format(self._description))
        next_check = datetime.now() + timedelta(seconds=5)
        while self._running:
            if datetime.now() > next_check:
                current = self._current
                if current is not None:
                    self._handle(current)
                next_check = self._next_check
            time.sleep(1)

    def stop(self):
        self._running = False

    @property
    def _next_check(self):
       return datetime.now() + timedelta(seconds=self._interval) 

    def _handle(self, monitor):
        pass

    @property
    def _current(self):
        try:
            return find(self._qualify, self._candidates)
        except sqlalchemy.exc.InvalidRequestError:
            pass

    def _qualify(self, release):
        return True

    @property
    def _candidates(self):
        return self._queue

__all__ = []
