import re

import requests

from tek.config import lazy_configurable

from series.util import is_torrent, domain
from tek_utils.sharehoster import DownloaderFactory
from tek_utils.sharehoster.errors import InvalidHosterError, InvalidURLError


@lazy_configurable(daemon=['min_size'])
class Link(object):

    _dl_factory = DownloaderFactory()

    def __init__(self, url):
        self.url = url
        self._downloader = None
        self._checked_status = False
        self._dead = False
        self._dead_reason = 'nothing'
        self._too_small = False
        self._multipart = False

    @property
    def invalid(self):
        if not self._checked_status:
            self._check()
        return self._dead or self._too_small or self._multipart

    @property
    def good(self):
        return not self.invalid

    def _check(self):
        if not self.is_torrent:
            dl = self.downloader
            if not self._dead:
                self._too_small = dl.file_size < self._min_size
                self._multipart = re.search('part\d+\.rar$',
                                            dl.file_path or '')
        self._checked_status = True

    @property
    def downloader(self):
        if self._downloader is None and not self._dead:
            try:
                self._dead = True
                self._downloader = self._dl_factory(self.url)
                self._dead = False
            except InvalidHosterError:
                self._dead_reason = 'invalid hoster'
            except (InvalidURLError, requests.RequestException) as e:
                self._dead_reason = str(e)
        return self._downloader

    @property
    def is_torrent(self):
        return is_torrent(self.url)

    @property
    def status(self):
        pre = 'In' if self.invalid else ''
        post = ': {}'.format(self._reason) if self.invalid else ''
        return '{}valid link{} ({})'.format(pre, post, self.url)

    @property
    def _reason(self):
        reason = ''
        if self._dead:
            reason = 'dead ({})'.format(self._dead_reason)
        elif self._too_small:
            reason = 'too small ({})'.format(self.downloader.file_size_str)
        elif self._multipart:
            reason = 'multipart archive'
        return reason

    @property
    def domain(self):
        return domain(self.url)

__all__ = ['Link']
