import re

from tek.tools import find

from series.daemon.model.release import ReleaseFactory, ReleaseMonitor


class ReleasesFacade(object):

    def __init__(self, queue, session):
        self._queue = queue
        self._session = session

    def _find(self, matcher, multi=False):
        finder = filter if multi else find
        return finder(matcher, self._queue)

    def find_by_id(self, _id):
        return self._find(lambda r: r.id == _id)

    def find_by_metadata(self, series=None, season=None, episode=None):
        def matcher(monitor):
            release = monitor.release
            return (
                (not(series) or release.name == series) and
                (not(season) or int(release.season) == int(season)) and
                (not(episode) or int(release.episode) == int(episode))
            )
        return self._find(matcher)

    def filter_episode_repr(self, regex):
        rex = re.compile(regex, re.I)
        matcher = lambda m: rex.search(str(m.release))
        return list(self._find(matcher, multi=True))

    def __getitem__(self, slice):
        return self._queue[slice]

    def create(self, series, season, episode):
        fact = ReleaseFactory()
        enum = '{:0>2}x{:0>2}'.format(season, episode)
        title = '{}_{}'.format(series, enum)
        release = fact(title, series, '', '', enum=enum)
        monitor = ReleaseMonitor(release, len(self._queue), [])
        self._session.add_then_commit(monitor)
        self._queue.append(monitor)

    @property
    def pending_downloads(self):
        qualify = lambda r: not r.downloaded and not r.nuked
        return list(filter(qualify, self._queue))
