# -*- coding: utf-8 -*-

import requests

from tek.config import lazy_configurable
from tek import logger

from series import subsync
from series.daemon.handler import Handler
from series.subsync.errors import NoSubsForEpisode


@lazy_configurable(series=['series_dir'])
class Subsyncer(Handler):

    def __init__(self, queue, commit):
        super(Subsyncer, self).__init__(queue, commit, 5,
                                        'subtitle downloader')

    def _qualify(self, monitor):
        return (monitor.archived and not monitor.subtitles_downloaded and not
                getattr(monitor, 'subs_missing', False))

    def _handle(self, monitor):
        release = monitor.release
        text = 'Downloading subs for episode {!s}…'
        logger.info(text.format(release))
        try:
            sub = subsync.get_episode(release.name, release.season,
                                      release.episode)
        except NoSubsForEpisode as e:
            logger.error(e)
            monitor.subs_missing = True
        except requests.RequestException as e:
            logger.error(e)
        else:
            self._write_sub(monitor, sub)

    def _write_sub(self, monitor, sub):
        try:
            sub.write()
        except IOError as e:
            logger.error(e)
        else:
            monitor.subtitles_downloaded = True
            logger.info('Successfully downloaded subtitles.')

__all__ = ['Subsyncer']
