import threading

from tek.config import Configurations, lazy_configurable
from tek.run import SignalManager

from series.library.db import FileDatabase
from series.library.rest_api import RestApi
from series.library.library_facade import LibraryFacade
from series.library.player import Player


@lazy_configurable(library=['db_path', 'run_rest_api'])
class LibraryDaemon(threading.Thread):

    def __init__(self):
        super(LibraryDaemon, self).__init__()
        self._db = FileDatabase(self._db_path)
        self._library = LibraryFacade(self._db)
        self._player = Player(self._library)
        self._rest_api = RestApi(self._library, self._player)

    def run(self):
        self.register_sigint_handler()
        if self._run_rest_api:
            self._rest_api.start()

    def register_sigint_handler(self):
        SignalManager.instance.sigint(self._interrupt)

    def _interrupt(self, signum, frame):
        self._running = False
        if self._run_rest_api:
            self._rest_api.stop()
        self._db.session.commit()
        if self._rest_api.is_alive():
            self._rest_api.join()


def libd_cli():
    from series.library.config import reset_config
    reset_config()
    Configurations.parse_cli()
    LibraryDaemon().run()

__all__ = ['LibraryDaemon', 'libd_cli']
