import os

from sqlpharmacy.core import Database as SPDatabase

from sqlalchemy import Column, Integer, Boolean

from factory.alchemy import SQLAlchemyModelFactory

from series.daemon.errors import InvalidDBError


class Database(SPDatabase):

    def __init__(self, connection_string='sqlite:///', **kw):
        Database.register()
        args = dict(check_same_thread=False)
        super(Database, self).__init__(connection_string, connect_args=args,
                                       **kw)
        self.commit = self.session.commit
        self.create_tables()
        SQLAlchemyModelFactory.FACTORY_SESSION = self.session

    def query(self, *a, **kw):
        return self.session.query(*a, **kw)


class FileDatabase(Database):

    def __init__(self, _path, **kw):
        self._path = _path
        self._check_path()
        connection_string = 'sqlite:///{}'.format(self._path)
        super(FileDatabase, self).__init__(connection_string, **kw)

    def _check_path(self):
        _dir = os.path.dirname(self._path)
        if _dir and not os.path.isdir(_dir):
            os.makedirs(_dir)
        if os.path.isdir(self._path):
            raise InvalidDBError('Is a directory!')

__all__ = ['Database', 'Column', 'Integer', 'Boolean']
