import functools

from factory.alchemy import SQLAlchemyModelFactory

from series.library.db import Database, Column, Integer, Boolean


@functools.total_ordering
@Database.many_to_one('Season', backref_name='episodes')
@Database.many_to_one('Series', backref_name='episodes')
class Episode(object, metaclass=Database.DefaultMeta):
    number = Column(Integer)
    title = Column(Integer)
    new = Column(Boolean)

    def __str__(self):
        templ = 'Episode {} of series "{}" season {}'
        return templ.format(self.number, self.series.formatted_name,
                            self.season.number)

    def __repr__(self):
        return '<Episode {}x{} ({})>'.format(self.season.number, self.number,
                                             self.series.formatted_name)

    def __lt__(self, other):
        if isinstance(other, Episode):
            return (self.series < other.series or self.season < other.season or
                    self.number < other.number)

    def __eq__(self, other):
        if isinstance(other, Episode):
            return (self.series == other.series or self.season == other.season
                    or self.number == other.number)

    def __hash__(self):
        return hash((self.series, self.season, self.number))

    @property
    def info(self):
        return dict(
            series=self.series.formatted_name,
            season=self.season.number,
            episode=self.number,
            title=self.title,
            new=self.new,
        )


class EpisodeFactory(SQLAlchemyModelFactory):
    FACTORY_FOR = Episode
    number = ''
    title = ''
    series = None
    season = None
    new = True

__all__ = ['Episode', 'EpisodeFactory']
