import functools

from factory.alchemy import SQLAlchemyModelFactory

from sqlpharmacy.core import Database

from sqlalchemy import Column, Integer


@functools.total_ordering
@Database.many_to_one('Series', backref_name='seasons')
class Season(object, metaclass=Database.DefaultMeta):
    number = Column(Integer)

    def __str__(self):
        return 'Season {} of series "{}"'.format(self.number, self.series.name)

    def __repr__(self):
        return '<Season {} ({})>'.format(self.number, self.series.name)

    def __lt__(self, other):
        if isinstance(other, Season):
            return (self.series < other.series or self.number < other.number)

    def __eq__(self, other):
        if isinstance(other, Season):
            return (self.series == other.series or self.number == other.number)

    def __hash__(self):
        return hash((self.series, self.number))

    @property
    def info(self):
        return dict(
            series=self.series.formatted_name,
            number=self.number,
            new_count=self.new_count,
        )

    @property
    def new_count(self):
        return len([e for e in self.episodes if e.new])


class SeasonFactory(SQLAlchemyModelFactory):
    FACTORY_FOR = Season
    number = ''
    series = None

__all__ = ['Season', 'SeasonFactory']
