import time
import threading
from datetime import datetime


class PlaybackObserver(threading.Thread):

    def __init__(self, mplayer, library, episode):
        super(PlaybackObserver, self).__init__()
        self._mplayer = mplayer
        self._library = library
        self._episode = episode
        self._start_time = None
        self._finish_time = None
        self._stopped = False

    def run(self):
        self._start_time = datetime.now()
        while self._running:
            time.sleep(1)
        self._finish()

    def stop(self):
        self._stopped = True

    def _finish(self):
        self._finish_time = datetime.now()
        self._library.add_watch_event(self._episode.series,
                                      self._episode.season,
                                      self._episode.number, self._start_time,
                                      self._finish_time)

    @property
    def _running(self):
        ''' Test whether the same episode is still loaded in mplayer.
        Either it has been terminated manually, which means that stop()
        has been called, or the episode is at its end, in which case an
        arbitrary attribute is checked (here length), as there seems to
        be no indicator for that.
        '''
        return not self._stopped and self._mplayer.length is not None

__all__ = ['PlaybackObserver']
