import os

from mplayer import Player as MPlayer

from tek.config import lazy_configurable

from series.library.playback_observer import PlaybackObserver


@lazy_configurable(library=['mplayer_display', 'mplayer_path'])
class Player(object):

    def __init__(self, library):
        self._library = library
        os.environ['DISPLAY'] = self._mplayer_display
        MPlayer.exec_path = self._mplayer_path
        self._mplayer = MPlayer(args=('-fs',))
        self._episode = None
        self._observer = None

    @property
    def episode(self):
        return self._episode

    @episode.setter
    def episode(self, epi):
        self.stop()
        self._episode = epi

    def start(self):
        self._observer = PlaybackObserver(self._mplayer, self._library,
                                          self._episode)
        self._mplayer.loadfile(self._file)
        self._observer.start()

    def toggle_pause(self):
        self._mplayer.pause()

    def seek(self, value):
        # TODO use Step()
        new_value = self.elapsed + value
        new_value = max(0, new_value)
        new_value = min(self.elapsed, new_value)
        self._mplayer.time_pos = new_value

    def stop(self):
        if self._observer is not None:
            self._observer.stop()
            self._mplayer.stop()
            self._observer = None

    @property
    def elapsed(self):
        return self._mplayer.time_pos

    @property
    def remaining(self):
        return self.length - self.elapsed

    @property
    def length(self):
        return self._mplayer.length

    @property
    def _file(self):
        return self._library.episode_path(self.episode)

__all__ = ['Player']
