import threading
import operator

from tek.config import lazy_configurable

from series.rest_api import RestApi as RestApiBase, json_response


@lazy_configurable(library=['rest_api_host', 'rest_api_port'])
class RestApi(RestApiBase):

    def __init__(self, library, player):
        RestApiBase.__init__(self, 'library')
        self._library = library
        self._player = player

    def setup_routes(self):
        self.app.add_url_rule('/series', 'series', self._series)
        self.app.add_url_rule('/series/<name>', 'series_data',
                              self._series_data)
        self.app.add_url_rule('/episode/new', 'new_episodes',
                              self._new_episodes)
        self.app.add_url_rule('/episode/recently_watched/<int:days>',
                              'recently_watched_episodes',
                              self._recently_watched_episodes)
        path = '/series/<series>/<int:season>/<int:episode>/watch'
        self.app.add_url_rule(path, 'watch', self._watch, methods=['PUT'])
        self.app.add_url_rule('/player/pause', 'toggle_pause',
                              self._player_toggle_pause, methods=['PUT'])
        self.app.add_url_rule('/player/stop', 'stop_player',
                              self._player_stop, methods=['PUT'])
        self.app.add_url_rule('/player/seek/forward/<int:value>',
                              'seek_forward', self._player_seek_forward,
                              methods=['PUT'])
        self.app.add_url_rule('/player/seek/backward/<int:value>',
                              'seek_backward', self._player_seek_backward,
                              methods=['PUT'])
        self.app.add_url_rule('/collection/<_id>/new', 'scan',
                              self._scan, methods=['PUT'])

    @json_response
    def _series(self):
        series = self._library.all_series
        info = [seri.info for seri in series]
        return sorted(info, key=operator.itemgetter('name'))

    @json_response
    def _series_data(self, name):
        data = {}
        series = self._library.series(name).info
        seasons = [s.info for s in self._library.seasons(series=name)]
        data['series'] = series
        data['seasons'] = sorted(seasons, key=operator.itemgetter('number'))
        return data

    @json_response
    def _new_episodes(self):
        episodes = self._library.new_episodes
        return [epi.info for epi in episodes]

    @json_response
    def _recently_watched_episodes(self, days):
        episodes = self._library.recently_watched_episodes(days=days)
        return [epi.info for epi in episodes]

    @json_response
    def _watch(self, series, season, episode):
        epi = self._library.episode(series, season, episode)
        if epi:
            self._player.episode = epi
            self._player.start()
            return 'success'
        else:
            return 'Episode not found!'

    @json_response
    def _player_toggle_pause(self):
        self._player.toggle_pause()

    @json_response
    def _player_seek_forward(self, value):
        self._player.seek(value)

    @json_response
    def _player_seek_backward(self, value):
        self._player.seek(-value)

    @json_response
    def _player_stop(self):
        self._player.stop()

    @json_response
    def _scan(self, _id):
        if _id == 'all':
            worker = threading.Thread(target=self._library.scan)
            worker.start()
            return 'Scanning initiated.'
        return 'Not yet implemented.'

__all__ = ['RestApi']
