import abc
import threading
from functools import wraps

import requests

from flask import Flask, jsonify, request

from tek import logger


def json_response(func):
    @wraps(func)
    def wrapper(*a, **kw):
        return jsonify(dict(response=func(*a, **kw)))
    return wrapper


class RestApi(threading.Thread, metaclass=abc.ABCMeta):

    def __init__(self, name):
        threading.Thread.__init__(self)
        self.app = Flask(name)

    def run(self):
        logger.info('Starting REST API.')
        self.setup_routes()
        self.app.run(host=self._rest_api_host, port=self._rest_api_port)

    def _shutdown(self):
        func = request.environ.get('werkzeug.server.shutdown')
        func()
        return 'server shut down.'

    def stop(self):
        self.app.add_url_rule('/shutdown', 'shutdown', self._shutdown,
                              methods=['POST'])
        uri = 'http://localhost:{}/shutdown'.format(self._rest_api_port)
        requests.post(uri)

    @abc.abstractmethod
    def setup_routes(self):
        pass

__all__ = ['RestApi']


__all__ = []
