import time

import sure
from flexmock import flexmock, flexmock_teardown

from tek import Configurations

from series.daemon.model.release import ReleaseMonitor
from series.daemon.rest_api import RestApi
from series.daemon.model.link import Link
from series.daemon.config import reset_config as d_reset_config
from series.client.config import reset_config as c_reset_config
from series.client.cli import HTTPCLI
from series.client.rest_api import ApiClient
from tests.daemon.db import _DBTestMixin
from tests import Spec


class HttpCli_(_DBTestMixin, Spec, ):

    def setup(self, *a, **kw):
        super(HttpCli_, self).setup()
        Configurations.override('daemon', rest_api_port=60123)
        Configurations.override('client', rest_api_port=60123)
        self._api = RestApi(self._queue, self._commit, self._session)
        self._api.app.config['TESTING'] = True
        self._api.start()
        import time
        time.sleep(1)

    def teardown(self, *a, **kw):
        self._api.stop()
        self._api.join()

    def it_should_retrieve_data_from_the_service(self):
        search_regex = 'S.*4'
        expected = [['2', 'Series1 4x2']]
        Configurations.override('client', cli_cmd=['list'],
                                cli_params=[search_regex])
        cli = HTTPCLI()
        response = cli.run()
        response.should.equal(expected)


class ApiClient_(_DBTestMixin, Spec, ):

    def setup(self, *a, **kw):
        super(ApiClient_, self).setup()
        Configurations.override('daemon', rest_api_port=60123)
        Configurations.override('client', rest_api_port=60123)
        self._api = RestApi(self._queue, self._commit, self._session)
        self._api.app.config['TESTING'] = True
        self._api.start()
        self._client = ApiClient()
        time.sleep(1)

    def teardown(self, *a, **kw):
        self._api.stop()
        self._api.join()
        flexmock_teardown()

    def list(self):
        result = self._client.list('S.*1')
        expected = [['1', 'Series1 1x5'], ['2', 'Series1 4x2']]
        result.should.equal(expected)

    def explain(self):
        (flexmock(Link).should_receive('_reason')
         .and_return('dead (invalid hoster)')
         )
        (flexmock(Link).should_receive('invalid')
         .and_return(True)
         )
        result = self._client.explain()
        expected = ['Series1 1x5 is not downloadable because it has no valid'
                    ' links:\n\tInvalid link: dead (invalid hoster)'
                    ' (http://foo.bum/file1a)\n\tInvalid link: dead (invalid'
                    ' hoster) (http://foo.bum/file1b).',
                    'Series1 4x2 is not downloadable because it has no valid'
                    ' links:\n\tInvalid link: dead (invalid hoster)'
                    ' (http://foo.bum/file2a)\n\tInvalid link: dead (invalid'
                    ' hoster) (http://foo.bum/file2b).',
                    'Series2 2x7 is not downloadable because it has no valid'
                    ' links:\n\tInvalid link: dead (invalid hoster)'
                    ' (http://foo.bum/file3a)\n\tInvalid link: dead (invalid'
                    ' hoster) (http://foo.bum/file3b).']
        result.should.equal(expected)

    def add_link_metadata(self):
        url = 'http://foo.bum/filefoo'
        series = 'series1'
        season = '4'
        episode = '2'
        result = self._client.add_link(url, series, season, episode)
        expected = 2
        result.should.equal(expected)

    def add_link_url(self):
        url = 'http://foo.bum/series1_s04e2.1080p'
        result = self._client.add_link(url)
        expected = 2
        result.should.equal(expected)

    def mark_downloaded_id(self):
        _id = '2'
        result = self._client.mark_downloaded(_id)
        expected = True
        result.should.equal(expected)

    def create_a_release(self):
        self._client.create_release('series_name', 5, 3)
        release = self._queue[-1].release
        release.title.should.equal('series_name_05x03')
        release.season.should.equal(5)
        release.episode.should.equal(3)

    def create_existing_release(self):
        result = self._client.create_release('series1', 4, 2)
        result.should.equal('Release already exists')
