import os
import shutil

import sure
from flexmock import flexmock

from tek import Configurations
from tek import logger

from series.daemon.archiver import Archiver
from series.daemon.errors import ArchiverError
from tests.daemon.db import _DBTestMixin
from tests import temp_file, temp_dir, Spec


class Archiver_(_DBTestMixin, Spec, ):

    def setup(self, *a, **kw):
        super(Archiver_, self).setup(*a, **kw)
        path_template = '{name}_{season}/{name}_{season}x{episode}.{ext}'
        self._series_dir = temp_dir('series', 'archiver', 'series')
        Configurations.override('series', series_dir=self._series_dir,
                                path_template=path_template)
        self._dest_path = os.path.join(temp_dir(self._series_dir, 'series1_1'),
                                       'series1_1x5.mkv')
        self._download_dir = temp_dir('series', 'archiver', 'download')
        self._archiver = Archiver(self._queue, self._session.commit)
        self._monitor = self._queue[0]

    def format_a_series_path(self):
        path = self._archiver._format_path(self._queue[0])
        path.should.be(self._dest_path)

    def store_a_file(self):
        monitor = self._queue[0]
        download_path = os.path.join(self._download_dir, 'store_test')
        with open(download_path, 'w') as _file:
            _file.write('success')
        monitor.download_path = download_path
        self._archiver._store(monitor, self._dest_path)
        with open(self._dest_path) as _file:
            _file.read().should.equal('success')

    class check_downloaded_file(object):

        def setup(self):
            self._monitor.download_path = os.path.join(self._download_dir,
                                                       'series1_1x5.mkv')

        def download_missing(self):
            (self._archiver._check_downloaded_file
             .when.called_with(self._monitor, self._dest_path)
             .should.throw(ArchiverError))
            self._monitor.archived.should.be.false

        def download_missing_and_archived_existing(self):
            with open(self._dest_path, 'w') as _file:
                _file.write('success')
            (self._archiver._check_downloaded_file
             .when.called_with(self._monitor, self._dest_path)
             .should.throw(ArchiverError))
            self._monitor.archived.should.be.true

        def download_existing(self):
            with open(self._monitor.download_path, 'w') as _file:
                _file.write('success')
            (self._archiver._check_downloaded_file
             .when.called_with(self._monitor, self._dest_path)
             .should_not.throw(ArchiverError))
            self._monitor.archived.should.be.false

    class check_target_dir(object):

        def existing_file(self):
            _path = temp_file('check_target_dir')
            with open(_path, 'w') as _file:
                _file.write('success')
            (self._archiver._check_target_dir.when.called_with(_path)
             .should.throw(ArchiverError))

        def existing_dir(self):
            _path = temp_dir('check_target_dir')
            (self._archiver._check_target_dir.when.called_with(_path)
             .should_not.throw(ArchiverError))

        def nonexisting(self):
            _path = temp_dir('check_target_dir')
            shutil.rmtree(_path)
            (self._archiver._check_target_dir.when.called_with(_path)
             .should_not.throw(ArchiverError))

__all__ = ['Archiver_']
