import os
import shutil

import httpretty
import sure
from flexmock import flexmock, flexmock_teardown

from tek import Configurations, ConfigClient

from tek_utils import sharehoster
from tek_utils.sharehoster.common import Downloader

from series.daemon.download import DownloadManager
from series.daemon.config import reset_config
from tests.daemon.db import _DBTestMixin
from tests.daemon.support.http import _setup_http_mocks
from tests import temp_dir, Spec


class DownloadManager_(_DBTestMixin, Spec, ):

    def setup(self, *a, **kw):
        reset_config(reset_parent=True)
        (flexmock(sharehoster.DownloaderFactory)
            .should_receive('__call__')
            .replace_with(Downloader))
        self._tmpdir = temp_dir('daemon', 'download')
        Configurations.override('daemon', download_dir=self._tmpdir)
        Configurations.override('common', link_checker_url=None)
        super(DownloadManager_, self).setup(*a, **kw)

    def teardown(self, *a, **kw):
        flexmock_teardown()
        shutil.rmtree(self._tmpdir, ignore_errors=True)
        super(DownloadManager_, self).teardown(*a, **kw)

    @httpretty.activate
    def it_should_download_a_file(self):
        _setup_http_mocks()
        outfile = os.path.join(ConfigClient('daemon')('download_dir'),
                               'file1a')
        self._downloader = DownloadManager(self._queue, self._commit)
        self._downloader._download_sharehoster(self._queue[0])
        with open(outfile) as _file:
            _file.read().should.equal('SUCCESS')
