import sure
from flexmock import flexmock

from series.daemon.feed_poller import FeedPoller
from series.daemon.model.feed_entry import FeedEntryFactory

from tests.daemon.db import _DBTestMixin
from tests import fixture_path, Spec


class FeedPoller_(_DBTestMixin, Spec):

    def parse_an_rss_feed(self):
        flexmock(FeedPoller).should_receive('_reconstruct_entries')
        fname = fixture_path('daemon', 'wrzko.rss')
        feed_poller = FeedPoller(self._db.session, self._queue, self._commit)
        feed_poller.load_from_xml_file(fname)
        entries = feed_poller._rss.entries
        entries.should.have.length_of(5)
        entries[0].should.be.a('feedparser.FeedParserDict')

    def discard_seen_entries(self):
        fname = fixture_path('daemon', 'wrzko.rss')
        feed_poller = FeedPoller(self._db.session, self._queue, self._commit)
        feed_poller.load_from_xml_file(fname)
        feed_poller._feed_entries.should.have.length_of(5)
        feed_poller.load_from_xml_file(fname)
        feed_poller._feed_entries.should.have.length_of(0)

    def create_monitor(self):
        (flexmock(FeedEntryFactory).should_receive('_domain')
            .and_return('wrzko'))
        feed_factory = FeedEntryFactory('')
        feed_poller = FeedPoller(self._db.session, self._queue, self._commit)
        title1 = ('series.name.s99e11.episode.title')
        title2 = ('series.name.s99e11.episode.title2')
        entry1 = feed_factory._release_from_title(title1)
        entry2 = feed_factory._release_from_title(title2)
        feed_poller._create_monitor(entry1)
        self._queue[-1].nuked.should_not.be.ok
        feed_poller._create_monitor(entry2)
        self._queue[-2].nuked.should.be.ok
