import re

import sure
from flexmock import flexmock, flexmock_teardown

from series.daemon.model.feed_entry import FeedEntryFactory
from tests import Spec


class FeedEntryFactory_(Spec):

    class wrzko:

        def setup(self):
            (flexmock(FeedEntryFactory).should_receive('_domain')
             .and_return('wrzko'))
            self._subject = FeedEntryFactory('')

        def teardown(self):
            flexmock_teardown()

        def it_should_parse_a_title(self):
            title = ('series.name.s99e11.episode.title5x3s5e1'
                     '.1080p.type.codec-group')
            feed_entry = self._subject._release_from_title(title)
            release = feed_entry.release
            release.is_series.should.be.ok
            release.title.should.be(title)
            release.name.should.be('series_name')
            release.codec.should.be('codec')
            release.season.should.be('99')
            release.episode.should.be('11')
            release.resolution.should.be('1080p')
            release.is_hd.should.be.ok
            release.group.should.be('group')
            feed_entry.is_fix.shouldnot.be.ok

        def it_should_parse_a_title_without_title(self):
            title = 'Series.Name.S07E11.720p.HDTV.x264-IMMERSE'
            feed_entry = self._subject._release_from_title(title)
            release = feed_entry.release
            release.is_series.should.be.ok
            release.name.should.be('series_name')
            release.episode.should.be('12')

        def it_should_create_a_search_string(self):
            title = 'Series.Name.S05E10.PROPER.720p.HDTV.x264-ORENJI'
            feed_entry = self._subject._release_from_title(title)
            strng = '3968699907/series.name_s05e10.720p_hdtv.x264-orenji'
            re.search(feed_entry.release.search_string, strng).should.be.ok

        def it_should_compare_releases(self):
            title = 'Series.Name.S01E10.720p.HDTV.x264-IMMERSE'
            r1 = self._subject._release_from_title(title).release
            r2 = self._subject._release_from_title(title).release
            r1.should.equal(r2)

        def it_should_detect_a_fix(self):
            repack = 'Series.Name.S01E10.REPACK.720p.HDTV.x264-IMMERSE'
            entry = self._subject._release_from_title(repack)
            entry.release.name.should.equal('series_name')
            entry.is_fix.should.be.ok

        def it_should_not_choke_on_a_minus(self):
            bb = 'Series-Name.S09E10.720p.HDTV.x264-IMMERSE'
            entry = self._subject._release_from_title(bb)
            entry.release.name.should.equal('series-name')

        def accept_double_episodes(self):
            title = ('series.name.s99e10e11.episode.title5x3s5e1'
                     '.1080p.type.codec-group')
            feed_entry = self._subject._release_from_title(title)
            release = feed_entry.release
            release.is_series.should.be.ok
            release.title.should.be(title)
            release.name.should.be('series_name')
            release.codec.should.be('codec')
            release.season.should.be('99')
            release.episode.should.be('10')
            release.resolution.should.be('1080p')
            release.is_hd.should.be.ok
            release.group.should.be('group')
            feed_entry.is_fix.shouldnot.be.ok

    class ezrss:

        def setup(self):
            (flexmock(FeedEntryFactory).should_receive('_domain')
             .and_return('ezrss'))
            self._subject = FeedEntryFactory('')

        def teardown(self):
            flexmock_teardown()

        def it_should_parse_a_title(self):
            title = 'series_name 99x11 [1080P - RTYPE - GROUP]'
            feed_entry = self._subject._release_from_title(title)
            release = feed_entry.release
            release.is_series.should.be.ok
            release.title.should.be(title)
            release.name.should.be('series_name')
            release.season.should.be('99')
            release.episode.should.be('11')
            release.resolution.should.be('1080p')
            release.is_hd.should.be.ok
            release.group.should.be('group')
            feed_entry.is_fix.shouldnot.be.ok
