import sure
from flexmock import flexmock, flexmock_teardown

from flask import json

from tek import Configurations, logger

from series.daemon.rest_api import RestApi
from series.daemon.model.link import Link
from tests.daemon.db import _DBTestMixin
import series.daemon.config
from tests import Spec


class RestApi_(_DBTestMixin, Spec, ):

    def setup(self, *a, **kw):
        super(RestApi_, self).setup()
        self._api = RestApi(self._queue, self._commit, self._session)
        self._api.app.config['TESTING'] = True
        self._client = self._api.app.test_client()
        self._api.setup_routes()

    def teardown(self):
        flexmock_teardown()

    def it_should_explain_downloadable_status(self):
        (flexmock(Link).should_receive('_reason')
         .and_return('dead (invalid hoster)')
         )
        (flexmock(Link).should_receive('invalid')
         .and_return(True)
         )
        expected = ['Series1 1x5 is not downloadable because it has no valid'
                    ' links:\n\tInvalid link: dead (invalid hoster)'
                    ' (http://foo.bum/file1a)\n\tInvalid link: dead (invalid'
                    ' hoster) (http://foo.bum/file1b).',
                    'Series1 4x2 is not downloadable because it has no valid'
                    ' links:\n\tInvalid link: dead (invalid hoster)'
                    ' (http://foo.bum/file2a)\n\tInvalid link: dead (invalid'
                    ' hoster) (http://foo.bum/file2b).',
                    'Series2 2x7 is not downloadable because it has no valid'
                    ' links:\n\tInvalid link: dead (invalid hoster)'
                    ' (http://foo.bum/file3a)\n\tInvalid link: dead (invalid'
                    ' hoster) (http://foo.bum/file3b).']
        request = self._client.get('/release/explain')
        response = json.loads(next(request.response))['response']
        response.should.equal(expected)

    def create_a_release(self):
        self._client.post('/release/series_name/5/3')
        release = self._queue[-1].release
        release.title.should.equal('series_name_05x03')
        release.season.should.equal(5)
        release.episode.should.equal(3)

    def create_existing_release(self):
        request = self._client.post('/release/series1/4/2')
        response = json.loads(next(request.response))['response']
        response.should.equal('Release already exists')

__all__ = ['RestApi_']
