import os

import sure
from flexmock import flexmock

from tek import Configurations, logger

from series.daemon.subsyncer import Subsyncer
from series.subsync.errors import NoSubsForEpisode
from series.subsync import Subtitle
from series import subsync, SubtitleMetadata
from series.daemon import config as d_config
from series.subsync import config as s_config

from tests.daemon.db import _DBTestMixin
from tests import temp_dir, Spec


class Subsyncer_(_DBTestMixin, Spec, ):

    def setup(self, *a, **kw):
        Configurations.clear()
        d_config.reset_config(reset_parent=True)
        s_config.reset_config()
        super(Subsyncer_, self).setup(*a, **kw)
        self._subject = Subsyncer(self._queue, self._commit)
        self._monitor = self._queue[0]

    def handle_an_existing_subtitle(self):
        flexmock(subsync).should_receive('get_episode')
        flexmock(Subsyncer).should_receive('_write_sub').once()
        self._monitor = self._queue[0]
        self._subject._handle(self._monitor)
        getattr(self._monitor, 'subs_missing', False).should.be.false

    def handle_a_missing_subtitle(self):
        release = self._monitor.release
        (flexmock(subsync)
         .should_receive('get_episode')
         .and_raise(NoSubsForEpisode, release.name, release.season,
                    release.episode))
        flexmock(Subsyncer).should_receive('_write_sub').never()
        self._subject._handle(self._monitor)
        self._monitor.subs_missing.should.be.true

    def write_a_subtitle(self):
        release = self._monitor.release
        name = release.name
        season = release.season
        episode = release.episode
        fname = '{}_{:0>2}x{:0>2}.srt'.format(name, season, episode)
        Configurations.override('series', series_dir=temp_dir('daemon',
                                                              'subsyncer',
                                                              'series'))
        info = SubtitleMetadata(name, season, episode)
        subtitle = Subtitle(info, 'group', 'dummy', False)
        flexmock(subtitle).should_receive('content').and_return('success')
        self._subject._write_sub(self._monitor, subtitle)
        sub_path = os.path.join(temp_dir('daemon', 'subsyncer', 'series'),
                                name, 's{}'.format(season), 'sub', fname)
        with open(sub_path) as _file:
            _file.read().should.equal('success')

__all__ = ['Subsyncer_']
