import shutil
import os
import re

import sure
from flexmock import flexmock, flexmock_teardown

from tek import Configurations
from tek.user_input import input_queue

from tek_utils.extract import config as e_config

from series.handle_episode import EpisodeHandler, config
from series.store_episode import config as st_config
from series.subsync import config as ss_config
from series import subsync

from tests import load_fixture, Spec


class EpisodeTest(Spec):

    def setup(self):
        Configurations.clear()
        Configurations.enable_lazy_class_attr = False
        config.reset_config(register_files=False, reset_parent=True)
        e_config.reset_config()
        st_config.reset_config()
        ss_config.reset_config()
        Configurations.override('subsync', base_url='http://fakesubs.com')
        self._data_dir = os.path.join(os.path.dirname(__file__),
                                      os.path.pardir, '_data')
        self._sub = lambda *p: os.path.join(self._data_dir, *p)
        self._temp = self._sub('handle_episode', 'temp')
        self._series = self._sub('handle_episode', 'series')
        shutil.rmtree(self._temp, ignore_errors=True)
        shutil.rmtree(self._series, ignore_errors=True)
        os.makedirs(self._temp)
        os.makedirs(self._series)
        Configurations.override('series', temp_dir=self._temp,
                                series_dir=self._series)

    def teardown(self):
        flexmock_teardown()

    def test_dir(self):
        sub_page = load_fixture('subsync', 'game_of_thrones_1x2_sub_page')
        mock = flexmock(subsync.CookieHTTPHandler)
        (mock.should_receive('get')
         .with_args('http://fakesubs.com/serie/game_of_thrones/1/2/1')
         .and_return(sub_page))
        (mock.should_receive('get')
         .with_args('http://fakesubs.com/serie/game_of_thrones/2/2/1')
         .and_return(sub_page))
        (mock.should_receive('get')
         .with_args('http://fakesubs.com/serie/dexter/5/99/1')
         .and_return(sub_page))
        (mock.should_receive('get').with_args(re.compile('original'))
         .and_return(''))
        input_queue.push('q', 'y', 'q')
        os.mkdir(os.path.join(self._series, 'dexter'))
        os.mkdir(os.path.join(self._series, 'game_of_thrones'))
        os.mkdir(os.path.join(self._series, 'subtitle'))
        os.mkdir(os.path.join(self._series, 'title'))
        handler = EpisodeHandler(pretend_delete=True)
        handler.handle_episodes([self._sub('handle_episode')])
        epis = sorted(handler._episodes, key=lambda f: f.filename)
        epis[0].valid.should.be.false
        out_file_path = os.path.join(self._series, 'title', 's1', 'sub',
                                     'title_01x02.srt')
        os.path.isfile(out_file_path).should.be.true
        epis[3].complete.should.be.false
        shutil.rmtree(self._temp, ignore_errors=True)
        shutil.rmtree(self._series, ignore_errors=True)
