import os
from datetime import datetime, timedelta

import spec
import sure
from flexmock import flexmock

from tek import Configurations, logger
from tek.test import temp_file, Spec

from series.library.model.watch_event import WatchEvent
from series.library.library_facade import LibraryFacade
from series.library.config import reset_config

from tests.library.db import _DBTestMixin
from tests._fixtures.library import create_episodes


class LibraryFacade_(_DBTestMixin, Spec, ):

    def setup(self, *a, **kw):
        reset_config()
        super(LibraryFacade_, self).setup(*a, **kw)
        episodes = [
            ['series1', 2, 1, 0],
            ['series2', 1, 1, 0],
            ['series1', 2, 2, 1],
        ]
        self._collections, self._episodes = create_episodes(['lib1', 'lib2'],
                                                            episodes)
        self._lib = LibraryFacade(self._db)
        self._lib.scan()

    def scan_collections(self):
        series = self._lib.series('series1')
        episode = self._lib.episode(series, 2, 1)
        episode.series.should.equal(series)
        episode.season.should.equal(self._lib.season(series=series, number=2))

    def determine_episode_path(self):
        _file = 'series1_2x2.mkv'
        _path = temp_file(self._collections[1], 'series1', '2', _file)
        episode = self._lib.episode(series='series1', season=2, number=2)
        self._lib.episode_path(episode).should.equal(_path)

    def add_watch_event(self):
        now = datetime.now()
        before = now - timedelta(minutes=22)
        self._lib.add_watch_event('series1', 2, 1, before, now)
        event = self._db.query(WatchEvent).first()
        (before - event.begin).total_seconds().should.be.lower_than(1)
        (now - event.end).total_seconds().should.be.lower_than(1)

    def query_recently_watched_episodes(self):
        now = datetime.now()
        first_begin = now - timedelta(hours=12)
        first_end = first_begin + timedelta(minutes=30)
        first_begin_2 = now - timedelta(hours=6)
        first_end_2 = first_begin_2 + timedelta(minutes=30)
        second_begin = now - timedelta(hours=24, minutes=15)
        second_end = second_begin + timedelta(minutes=30)
        third_begin = now - timedelta(hours=48)
        third_end = third_begin + timedelta(minutes=30)
        self._lib.add_watch_event('series1', 2, 1, first_begin, first_end)
        self._lib.add_watch_event('series1', 2, 1, first_begin_2, first_end_2)
        self._lib.add_watch_event('series1', 2, 2, second_begin, second_end)
        self._lib.add_watch_event('series2', 1, 1, third_begin, third_end)
        episodes = self._lib.recently_watched_episodes(days=1)
        episodes.should.have.length_of(1)
        episodes[0].series.canonical_name.should.equal('series1')
        episodes[0].number.should.equal(1)

__all__ = ['LibraryFacade_']
