import spec
import sure
from flexmock import flexmock

from flask import json

from tek import Configurations, logger
from tek.tools import find
from tek.test import Spec

from series.library.rest_api import RestApi
from series.library.player import Player
from series.library.library_facade import LibraryFacade
from series.library.config import reset_config
from tests.library.db import _DBTestMixin
from tests._fixtures.library import create_episodes


class RestApi_(_DBTestMixin, Spec, ):
    # TODO use factory_boy for creating files

    def setup(self, *a, **kw):
        reset_config()
        super(RestApi_, self).setup()
        episodes = [
            ['series1', 5, 1, 0],
            ['series1', 5, 2, 0],
            ['series1', 5, 3, 0],
            ['series1', 6, 4, 0],
            ['series2', 3, 1, 0],
            ['series2', 3, 2, 0],
        ]
        _, self._episodes = create_episodes(['rest_api'], episodes)
        self._lib = LibraryFacade(self._db)
        self._lib.scan()
        self._lib.episode('series1', 5, 3).new = False
        self._lib.episode('series2', 3, 2).new = False
        player = Player(self._lib)
        self._api = RestApi(self._lib, player)
        self._api.app.config['TESTING'] = True
        self._client = self._api.app.test_client()
        self._api.setup_routes()

    def query_new_episodes(self):
        request = self._client.get('episode/new')
        response = json.loads(next(request.response))['response']

        def is_epi(s, ss, e):
            return lambda el: (el['series'] == s and el['season'] == ss and
                               el['episode'] == e)
        find(is_epi('Series1', 5, 2), response).should.be.a(dict)
        find(is_epi('Series1', 5, 3), response).should.be.none
        find(is_epi('Series2', 3, 2), response).should.be.none

    def play_a_video(self):
        _path = self._episodes[1]
        flexmock(Player).should_receive('play').with_args(_path)
        request = self._client.put('series/series1/5/2/watch')
        response = json.loads(next(request.response))['response']
        response.should.equal('success')

    def query_series(self):
        request = self._client.get('/series')
        response = json.loads(next(request.response))['response']
        response[0]['name'].should.equal('Series1')
        response[0]['new_count'].should.equal(3)
        response[1]['new_count'].should.equal(1)

    def query_season(self):
        request = self._client.get('series/series1')
        response = json.loads(next(request.response))['response']
        response['name'].should.equal('Series1')
        response['seasons'][0]['new_count'].should.equal(2)
        response['seasons'][1]['new_count'].should.equal(1)

__all__ = ['RestApi_']
